#define _GNU_SOURCE
#include <unistd.h>
#include <errno.h>
#include <limits.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>

static int (*open_orig) (const char *pathname, int flags, ...) = NULL;

static int in_veroot(const char *pathname)
{
	char *veroot;

	/* Check for VEROOT */
	if ((veroot = getenv("VZTT_VEROOT")) == NULL)
		return 0;

	if (strcmp(veroot, "/") == 0)
		return 0;

	/* veroot and pathname prefix are the same
	   normalization guaranteed by vztt */
	if (strncmp(veroot, pathname, strlen(veroot)))
		return 0;

	return chroot(veroot) ? -1 : strlen(veroot);
}

#ifdef __i386__
int open64(const char *pathname, int flags, ...)
#else
int open(const char *pathname, int flags, ...)
#endif
{
	int mode = 0;
	char *opensym;
	int fd, shift = 0;

#ifdef __i386__
	opensym = "open64";
#else
	opensym = "open";
#endif

	if (open_orig == NULL)
		open_orig = dlsym(RTLD_NEXT, opensym);

	if (flags & O_CREAT) {
		va_list arg;
		va_start(arg, flags);
		mode = va_arg(arg, int);
		va_end(arg);
	}

	if ((shift = in_veroot(pathname)) == -1)
		return -1;

	fd = open_orig(pathname + shift, flags, mode);

	if (shift && chroot(".") == -1) {
		if (fd != -1)
			close(fd);
		return -1;
	}

	return fd;
}
